/**
 * Copyright (c) 2025 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#pragma once
#include "packetbase.h"

typedef struct {
  int32_t number;
} packetping_t;

/**
 * Creates a ping packet.
 * 
 * @param packet Pointer to the packet structure to initialize.
 */
void packetPingCreate(packet_t *packet);

/**
 * Validates a ping packet received FROM a client INTO a server.
 * 
 * @param packet Pointer to the packet structure to validate.
 * @param client Pointer to the server client structure.
 * @return ERROR_OK on success, or an error code on failure.
 */
errorret_t packetPingClientProcess(
  const packet_t *packet,
  client_t *client
);

/**
 * Handles a ping packet received FROM a client INTO a server.
 * 
 * @param packet Pointer to the packet structure to handle.
 * @param client Pointer to the server client structure.
 * @return ERROR_OK on success, or an error code on failure.
 */
errorret_t packetPingServerProcess(
  const packet_t *packet,
  serverclient_t *client
);