/**
 * Copyright (c) 2025 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#pragma once
#include "packet/packet.h"
#include <pthread.h>

#define PACKET_QUEUE_MAX_SIZE 512

typedef struct {
  packet_t packetsIn[PACKET_QUEUE_MAX_SIZE];
  uint32_t packetsInCount;
  packet_t packetsOut[PACKET_QUEUE_MAX_SIZE];
  uint32_t packetsOutCount;
  pthread_mutex_t lock;
} packetqueue_t;

/**
 * Initializes the packet queue.
 * 
 * @param queue Pointer to the packet queue structure.
 */
void packetQueueInit(packetqueue_t *queue);

/**
 * Pushes a packet into the inbound packet queue.
 * 
 * @param queue Pointer to the packet queue structure.
 * @param packet Pointer to the packet to be pushed.
 */
void packetQueuePushIn(packetqueue_t *queue, const packet_t *packet);

/**
 * Pushes a packet into the outbound packet queue.
 * 
 * @param queue Pointer to the packet queue structure.
 * @param packet Pointer to the packet to be pushed.
 */
void packetQueuePushOut(packetqueue_t *queue, const packet_t *packet);

/**
 * Pops a packet from the inbound packet queue.
 * 
 * @param queue Pointer to the packet queue structure.
 * @param packet Pointer to the packet to store the popped packet.
 * @return 1 if a packet was popped, 0 otherwise.
 */
int32_t packetQueuePopIn(packetqueue_t *queue, packet_t *packet);

/**
 * Pops a packet from the outbound packet queue.
 * 
 * @param queue Pointer to the packet queue structure.
 * @param packet Pointer to the packet to store the popped packet.
 * @return 1 if a packet was popped, 0 otherwise.
 */
int32_t packetQueuePopOut(packetqueue_t *queue, packet_t *packet);