/**
 * Copyright (c) 2025 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "packet.h"
#include "util/memory.h"
#include "assert/assert.h"

void packetWelcomeCreate(packet_t *packet) {
  packetInit(packet, PACKET_TYPE_WELCOME, PACKET_WELCOME_SIZE);
  memoryCopy(
    packet->data.welcome.dusk, PACKET_WELCOME_STRING, PACKET_WELCOME_SIZE
  );
}

errorret_t packetWelcomeClient(packet_t *packet) {
  assertNotNull(packet, "Packet is NULL");
  assertTrue(packet->type == PACKET_TYPE_WELCOME, "Packet type is not WELCOME");

  if(packet->length != PACKET_WELCOME_SIZE) {
    return error("Welcome packet length is not %d", PACKET_WELCOME_SIZE);
  }

  if(
    memoryCompare(
      packet->data.welcome.dusk, PACKET_WELCOME_STRING, PACKET_WELCOME_SIZE
    ) != 0
  ) {
    return error("Welcome packet data is not %s", PACKET_WELCOME_STRING);
  }

  return ERROR_OK;
}